use warnings;
use strict;

use Test::More;

use Keyword::Declare;



# Autogenerated type declaration tests (from bin/gen_types.pl)...

keyword named (ArrayIndexer $n) {}
keyword named (AssignmentOperator $n) {}
keyword named (Attributes $n) {}
keyword named (Comma $n) {}
keyword named (Document $n) {}
keyword named (HashIndexer $n) {}
keyword named (InfixBinaryOperator $n) {}
keyword named (LowPrecedenceInfixOperator $n) {}
keyword named (OWS $n) {}
keyword named (PostfixUnaryOperator $n) {}
keyword named (PrefixUnaryOperator $n) {}
keyword named (StatementModifier $n) {}
keyword named (NWS $n) {}
keyword named (Whitespace $n) {}
keyword named (Statement $n) {}
keyword named (Block $n) {}
keyword named (Comment $n) {}
keyword named (ControlBlock $n) {}
keyword named (Expression $n) {}
keyword named (Expr $n) {}
keyword named (Format $n) {}
keyword named (Keyword $n) {}
keyword named (Label $n) {}
keyword named (PackageDeclaration $n) {}
keyword named (Pod $n) {}
keyword named (SubroutineDeclaration $n) {}
keyword named (UseStatement $n) {}
keyword named (LowPrecedenceNotExpression $n) {}
keyword named (List $n) {}
keyword named (CommaList $n) {}
keyword named (Assignment $n) {}
keyword named (ConditionalExpression $n) {}
keyword named (Ternary $n) {}
keyword named (ListElem $n) {}
keyword named (BinaryExpression $n) {}
keyword named (PrefixPostfixTerm $n) {}
keyword named (Term $n) {}
keyword named (AnonymousArray $n) {}
keyword named (AnonArray $n) {}
keyword named (AnonymousHash $n) {}
keyword named (AnonHash $n) {}
keyword named (AnonymousSubroutine $n) {}
keyword named (Call $n) {}
keyword named (DiamondOperator $n) {}
keyword named (DoBlock $n) {}
keyword named (EvalBlock $n) {}
keyword named (Literal $n) {}
keyword named (Lvalue $n) {}
keyword named (ParenthesesList $n) {}
keyword named (ParensList $n) {}
keyword named (Quotelike $n) {}
keyword named (ReturnStatement $n) {}
keyword named (Typeglob $n) {}
keyword named (VariableDeclaration $n) {}
keyword named (VarDecl $n) {}
keyword named (Variable $n) {}
keyword named (Var $n) {}
keyword named (ArrayAccess $n) {}
keyword named (Bareword $n) {}
keyword named (BuiltinFunction $n) {}
keyword named (HashAccess $n) {}
keyword named (Number $n) {}
keyword named (Num $n) {}
keyword named (QuotelikeQW $n) {}
keyword named (QuotelikeQX $n) {}
keyword named (Regex $n) {}
keyword named (Regexp $n) {}
keyword named (ScalarAccess $n) {}
keyword named (String $n) {}
keyword named (Str $n) {}
keyword named (Substitution $n) {}
keyword named (QuotelikeS $n) {}
keyword named (Transliteration $n) {}
keyword named (QuotelikeTR $n) {}
keyword named (ContextualRegex $n) {}
keyword named (Heredoc $n) {}
keyword named (Integer $n) {}
keyword named (Int $n) {}
keyword named (Match $n) {}
keyword named (QuotelikeM $n) {}
keyword named (NullaryBuiltinFunction $n) {}
keyword named (OldQualifiedIdentifier $n) {}
keyword named (QuotelikeQ $n) {}
keyword named (QuotelikeQQ $n) {}
keyword named (QuotelikeQR $n) {}
keyword named (VString $n) {}
keyword named (VariableArray $n) {}
keyword named (VarArray $n) {}
keyword named (ArrayVar $n) {}
keyword named (VariableHash $n) {}
keyword named (VarHash $n) {}
keyword named (HashVar $n) {}
keyword named (VariableScalar $n) {}
keyword named (VarScalar $n) {}
keyword named (ScalarVar $n) {}
keyword named (VersionNumber $n) {}
keyword named (ContextualMatch $n) {}
keyword named (ContextualQuotelikeM $n) {}
keyword named (PositiveInteger $n) {}
keyword named (PosInt $n) {}
keyword named (QualifiedIdentifier $n) {}
keyword named (QualIdent $n) {}
keyword named (QuotelikeQR $n) {}
keyword named (VString $n) {}
keyword named (Identifier $n) {}
keyword named (Ident $n) {}

# End of autogenerated type translations

ok 1, 'Compiled';

done_testing();

